unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  clDC, clUploader, StdCtrls, clDCUtils, clMultiDC, clSingleDC;

type
  TMyAppForm = class(TForm)
    btnContinuable: TButton;
    clUploader: TclUploader;
    btnNonContinuable: TButton;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure btnContinuableClick(Sender: TObject);
    procedure btnNonContinuableClick(Sender: TObject);
    procedure clUploaderDataItemProceed(Sender: TObject;
      ResourceInfo: TclResourceInfo; AStateItem: TclResourceStateItem;
      CurrentData: PChar; CurrentDataSize: Integer);
  private
    FPrevHandleException: TExceptionEvent;
    procedure DoHandleException(Sender: TObject; E: Exception);
  end;

var
  MyAppForm: TMyAppForm;

implementation

{$R *.DFM}

procedure TMyAppForm.DoHandleException(Sender: TObject; E: Exception);
var
  IsContinuable: Boolean;
  Msg: TStrings;
begin
  IsContinuable := not (E is EAccessViolation);
  Msg := TStringList.Create();
  try
    if IsContinuable then
    begin
      Msg.Add('The program raised continuable exception and still can continue to operate.');
    end else
    begin
      Msg.Add('The program raised noncontinuable exception and will be closed.');
    end;
    Msg.Add(E.Message);
    Msg.Add(' ... some detailed program information ... ');
    Msg.Add('Would you like to send detailed error info to the prorgam vendor ? ');
    if MessageDlg(Msg.Text, mtInformation, [mbYes, mbNo], 0) = mrYes then
    begin
      Msg.SaveToFile(clUploader.LocalFile);
      clUploader.Start(IsContinuable);
    end;
  finally
    Msg.Free();
  end;
  if not IsContinuable then
  begin
    Halt;
  end;
end;

procedure TMyAppForm.FormCreate(Sender: TObject);
begin
  FPrevHandleException := Application.OnException;
  Application.OnException := DoHandleException;
end;

procedure TMyAppForm.FormDestroy(Sender: TObject);
begin
  Application.OnException := FPrevHandleException;
end;

procedure TMyAppForm.btnContinuableClick(Sender: TObject);
begin
  raise Exception.Create('Some errorneous situation');
end;

procedure TMyAppForm.btnNonContinuableClick(Sender: TObject);
begin
  //Access Violation exception emulator
  asm
    xor EAX, EAX; //here we zero CPU register 
    mov EAX, [EAX]; //here we try to access to memory with zero address 
  end;
end;

procedure TMyAppForm.clUploaderDataItemProceed(Sender: TObject;
  ResourceInfo: TclResourceInfo; AStateItem: TclResourceStateItem; CurrentData: PChar;
  CurrentDataSize: Integer);
begin
  Caption := 'MyAppForm. Bug Report Send - ' + IntToStr(AStateItem.ResourceState.BytesProceed)
    + ' of ' + IntToStr(ResourceInfo.Size) + ' bytes proceed.';
end;

end.
